@@ -139,8 +139,10 @@ int PKCS11_login(PKCS11_SLOT * slot, int so, const char *pin)
 
 	rv = CRYPTOKI_call(ctx, C_Login(priv->session,
 					so ? CKU_SO : CKU_USER,
-					(CK_UTF8CHAR *) pin, strlen(pin)));
-	CRYPTOKI_checkerr(PKCS11_F_PKCS11_LOGIN, rv);
+					(CK_UTF8CHAR *) pin,
+					pin ? strlen(pin) : 0));
+	if (rv && rv != CKR_USER_ALREADY_LOGGED_IN)  /* logged in -> OK   */
+		CRYPTOKI_checkerr(PKCS11_F_PKCS11_LOGIN, rv);
 	priv->loggedIn = 1;
 	return 0;
 }
@@ -175,9 +177,8 @@ int PKCS11_logout(PKCS11_SLOT * slot)
 int PKCS11_init_token(PKCS11_TOKEN * token, const char *pin, const char *label)
 {
 	PKCS11_SLOT_private *priv = PRIVSLOT(TOKEN2SLOT(token));
-	PKCS11_CTX_private *cpriv;
 	PKCS11_CTX *ctx = priv->parent;
-	int n, rv;
+	int rv;
 
 	if (!label)
 		label = "PKCS#11 Token";
@@ -187,6 +188,8 @@ int PKCS11_init_token(PKCS11_TOKEN * token, const char *pin, const char *label)
 	CRYPTOKI_checkerr(PKCS11_F_PKCS11_INIT_TOKEN, rv);
 
 	/* FIXME: how to update the token?
+	 * PKCS11_CTX_private *cpriv;
+	 * int n;
 	 * cpriv = PRIVCTX(ctx);
 	 * for (n = 0; n < cpriv->nslots; n++) {
 	 * 	if (pkcs11_check_token(ctx, cpriv->slots + n) < 0)
@@ -287,7 +290,7 @@ int PKCS11_generate_random(PKCS11_SLOT *slot, unsigned char *r,
 /*
  * Helper functions
  */
-int pkcs11_init_slot(PKCS11_CTX * ctx, PKCS11_SLOT * slot, CK_SLOT_ID id)
+static int pkcs11_init_slot(PKCS11_CTX * ctx, PKCS11_SLOT * slot, CK_SLOT_ID id)
 {
 	PKCS11_SLOT_private *priv;
 	CK_SLOT_INFO info;
@@ -335,7 +338,7 @@ void pkcs11_release_slot(PKCS11_CTX * ctx, PKCS11_SLOT * slot)
 	memset(slot, 0, sizeof(*slot));
 }
 
-int pkcs11_check_token(PKCS11_CTX * ctx, PKCS11_SLOT * slot)
+static int pkcs11_check_token(PKCS11_CTX * ctx, PKCS11_SLOT * slot)
 {
 	PKCS11_SLOT_private *priv = PRIVSLOT(slot);
 	PKCS11_TOKEN_private *tpriv;
@@ -350,7 +353,7 @@ int pkcs11_check_token(PKCS11_CTX * ctx, PKCS11_SLOT * slot)
 	token = slot->token;
 
 	rv = CRYPTOKI_call(ctx, C_GetTokenInfo(priv->id, &info));
-	if (rv == CKR_TOKEN_NOT_PRESENT) {
+	if (rv == CKR_TOKEN_NOT_PRESENT || rv == CKR_TOKEN_NOT_RECOGNIZED) {
 		OPENSSL_free(token);
 		slot->token = NULL;
 		return 0;
@@ -377,7 +380,7 @@ int pkcs11_check_token(PKCS11_CTX * ctx, PKCS11_SLOT * slot)
 	return 0;
 }
 
-void pkcs11_destroy_token(PKCS11_TOKEN * token)
+static void pkcs11_destroy_token(PKCS11_TOKEN * token)
 {
 	pkcs11_destroy_keys(token);
 	pkcs11_destroy_certs(token);
